/*\file PenaltySystemMatrices.c
 *\brief: build penalty matrices
 */

#include "./PenaltySystemMatrices.h"

void mexFunction( int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]){

	/*diverse: */
	int   noerr=1;

	/*output: */
	double kmax;

	/*input datasets: */
	Mat      Kgg=NULL;
	Vec      pg=NULL;
	DataSet* elements=NULL;
	DataSet* nodes=NULL;
	DataSet* loads=NULL;
	DataSet* materials=NULL;
	int         kflag,pflag;
	ParameterInputs* inputs=NULL;
	int               analysis_type;
	char*             analysis_type_string=NULL;
	int               sub_analysis_type;
	char*             sub_analysis_type_string=NULL;
	
	/*Boot module: */
	MODULEBOOT();

	/*checks on arguments on the matlab side: */
	CheckNumMatlabArguments(nlhs,NLHS,nrhs,NRHS,__FUNCT__,&PenaltySystemMatricesUsage);

	/*Input datasets: */
	FetchData((void**)&Kgg,NULL,NULL,KGGIN,"Matrix",NULL);
	FetchData((void**)&pg,NULL,NULL,PGIN,"Vector",NULL);
	FetchData((void**)&elements,NULL,NULL,ELEMENTS,"DataSet",NULL);
	FetchData((void**)&nodes,NULL,NULL,NODES,"DataSet",NULL);
	FetchData((void**)&loads,NULL,NULL,LOADS,"DataSet",NULL);
	FetchData((void**)&materials,NULL,NULL,MATERIALS,"DataSet",NULL);
	/*parameters: */
	FetchData((void**)&kflag,NULL,NULL,mxGetField(PARAMETERS,0,"kflag"),"Integer",NULL);
	FetchData((void**)&pflag,NULL,NULL,mxGetField(PARAMETERS,0,"pflag"),"Integer",NULL);

	FetchData((void**)&analysis_type_string,NULL,NULL,ANALYSIS,"String",NULL);
	analysis_type=AnalysisTypeAsEnum(analysis_type_string);

	FetchData((void**)&sub_analysis_type_string,NULL,NULL,SUBANALYSIS,"String",NULL);
	sub_analysis_type=AnalysisTypeAsEnum(sub_analysis_type_string);


	/*Fetch inputs: */
	inputs=new ParameterInputs;
	inputs->Init(INPUTS);

	/*!Generate stiffnesses from penalties: */
	PenaltySystemMatricesx(Kgg, pg,&kmax,elements,nodes,loads,materials,kflag,pflag,inputs,analysis_type,sub_analysis_type); 

	/*write output datasets: */
	WriteData(KGG,Kgg,0,0,"Matrix",NULL); 
	WriteData(PG,pg,0,0,"Vector",NULL); 
	WriteData(KMAX,&kmax,0,0,"Scalar",NULL); 
	
	/*Free ressources: */
	delete elements;
	delete nodes;
	delete loads;
	delete materials;
	delete inputs;
	MatFree(&Kgg);
	VecFree(&pg);
	xfree((void**)&analysis_type_string);
	xfree((void**)&sub_analysis_type_string);

	/*end module: */
	MODULEEND();
}

void PenaltySystemMatricesUsage(void)
{
	_printf_("\n");
	_printf_("   usage: [Kgg,pg] = %s(Kggin,pgin,eleemnts,nodes,loads,materials,params,inputs,analysis_type,sub_analysis_type);\n",__FUNCT__);
	_printf_("\n");
}
